@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2016, 2018 SAP SE. All rights reserved.
+ * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2021 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,27 +27,19 @@
 #define CPU_S390_VM_VM_VERSION_S390_HPP
 
 
+#include "runtime/abstract_vm_version.hpp"
 #include "runtime/globals_extension.hpp"
-#include "runtime/vm_version.hpp"
 
 class VM_Version: public Abstract_VM_Version {
 
  protected:
-// The following list contains the (approximate) announcement/availability
-// dates of the many System z generations in existence as of now which
-// implement the z/Architecture.
-//   z900: 2000-10
-//   z990: 2003-06
-//   z9:   2005-09
-//   z10:  2007-04
-//   z10:  2008-02
-//   z196: 2010-08
-//   ec12: 2012-09
-//   z13:  2015-03
-//
 // z/Architecture is the name of the 64-bit extension of the 31-bit s390
 // architecture.
 //
+// For information concerning the life span of the individual
+// z/Architecture models, please check out the comments/tables
+// in vm_version_s390.cpp
+
 // ----------------------------------------------
 // --- FeatureBitString Bits   0.. 63 (DW[0]) ---
 // ----------------------------------------------
@@ -55,7 +47,7 @@ class VM_Version: public Abstract_VM_Version {
 //                                        04826048260482604
 #define  StoreFacilityListExtendedMask  0x0100000000000000UL  // z9
 #define  ETF2Mask                       0x0000800000000000UL  // z900
-#define  CryptoFacilityMask             0x0000400000000000UL  // z990
+#define  CryptoFacilityMask             0x0000400000000000UL  // z990 (aka message-security assist)
 #define  LongDispFacilityMask           0x0000200000000000UL  // z900 with microcode update
 #define  LongDispFacilityHighPerfMask   0x0000300000000000UL  // z990
 #define  HFPMultiplyAndAddMask          0x0000080000000000UL  // z990
@@ -94,11 +86,8 @@ class VM_Version: public Abstract_VM_Version {
 #define  LoadStoreConditional2Mask      0x0000000000000400UL  // z13
 #define  CryptoExtension5Mask           0x0000000000000040UL  // z13
 // z13 end
-// Feature-DW[0] starts to fill up. Use of these masks is risky.
-#define  TestFeature1ImplMask           0x0000000000000001UL
-#define  TestFeature2ImplMask           0x0000000000000002UL
-#define  TestFeature4ImplMask           0x0000000000000004UL
-#define  TestFeature8ImplMask           0x0000000000000008UL
+#define  MiscInstrExt2Mask              0x0000000000000020UL  // z14
+#define  MiscInstrExt3Mask              0x0000000000000004UL  // z15
 // ----------------------------------------------
 // --- FeatureBitString Bits  64..127 (DW[1]) ---
 // ----------------------------------------------
@@ -107,7 +96,7 @@ class VM_Version: public Abstract_VM_Version {
 //                                        48260482604826048
 #define  TransactionalExecutionMask     0x0040000000000000UL  // ec12
 #define  CryptoExtension3Mask           0x0008000000000000UL  // z196
-#define  CryptoExtension4Mask           0x0004000000000000UL  // z196
+#define  CryptoExtension4Mask           0x0004000000000000UL  // z196 (aka message-security assist extension 4, for KMF, KMCTR, KMO)
 #define  DFPPackedConversionMask        0x0000800000000000UL  // z13
 // ----------------------------------------------
 // --- FeatureBitString Bits 128..192 (DW[2]) ---
@@ -116,6 +105,15 @@ class VM_Version: public Abstract_VM_Version {
 //                                        23344455666778889
 //                                        82604826048260482
 #define  VectorFacilityMask             0x4000000000000000UL  // z13, not avail in VM guest mode!
+#define  ExecutionProtectionMask        0x2000000000000000UL  // z14
+#define  GuardedStorageMask             0x0400000000000000UL  // z14
+#define  VectorEnhancements1Mask        0x0100000000000000UL  // z14
+#define  VectorPackedDecimalMask        0x0200000000000000UL  // z14
+#define  CryptoExtension8Mask           0x0000200000000000UL  // z14 (aka message-security assist extension 8, for KMA)
+#define  VectorEnhancements2Mask        0x0000080000000000UL  // z15
+#define  VectorPackedDecimalEnhMask     0x0000008000000000UL  // z15
+#define  CryptoExtension9Mask           0x0000001000000000UL  // z15 (aka message-security assist extension 9)
+#define  DeflateMask                    0x0000010000000000UL  // z15
 
   enum {
     _max_cache_levels = 8,    // As limited by ECAG instruction.
@@ -123,10 +121,18 @@ class VM_Version: public Abstract_VM_Version {
     _code_buffer_len = 2*256  // For feature detection code.
   };
   static unsigned long _features[_features_buffer_len];
-  static unsigned long _cipher_features[_features_buffer_len];
+  static unsigned long _cipher_features_KM[_features_buffer_len];
+  static unsigned long _cipher_features_KMA[_features_buffer_len];
+  static unsigned long _cipher_features_KMF[_features_buffer_len];
+  static unsigned long _cipher_features_KMCTR[_features_buffer_len];
+  static unsigned long _cipher_features_KMO[_features_buffer_len];
   static unsigned long _msgdigest_features[_features_buffer_len];
   static unsigned int  _nfeatures;
-  static unsigned int  _ncipher_features;
+  static unsigned int  _ncipher_features_KM;
+  static unsigned int  _ncipher_features_KMA;
+  static unsigned int  _ncipher_features_KMF;
+  static unsigned int  _ncipher_features_KMCTR;
+  static unsigned int  _ncipher_features_KMO;
   static unsigned int  _nmsgdigest_features;
   static unsigned int  _Dcache_lineSize;
   static unsigned int  _Icache_lineSize;
@@ -134,16 +140,19 @@ class VM_Version: public Abstract_VM_Version {
   static const char*   _model_string;
 
   static bool test_feature_bit(unsigned long* featureBuffer, int featureNum, unsigned int bufLen);
+  static int  get_model_index();
   static void set_features_string();
   static void print_features_internal(const char* text, bool print_anyway=false);
   static void determine_features();
   static long call_getFeatures(unsigned long* buffer, int buflen, int functionCode);
   static void set_getFeatures(address entryPoint);
+  static void clear_buffer(unsigned long* buffer, unsigned int len);
+  static void copy_buffer(unsigned long* to, unsigned long* from, unsigned int len);
   static int  calculate_ECAG_functionCode(unsigned int attributeIndication,
                                           unsigned int levelIndication,
                                           unsigned int typeIndication);
 
-  // Setting features via march=z900|z990|z9|z10|z196|ec12|z13|ztest commandline option.
+  // Setting features via march=z900|z990|z9|z10|z196|ec12|z13|z14|z15 commandline option.
   static void reset_features(bool reset);
   static void set_features_z900(bool reset = true);
   static void set_features_z990(bool reset = true);
@@ -152,8 +161,17 @@ class VM_Version: public Abstract_VM_Version {
   static void set_features_z196(bool reset = true);
   static void set_features_ec12(bool reset = true);
   static void set_features_z13(bool reset = true);
+  static void set_features_z14(bool reset = true);
+  static void set_features_z15(bool reset = true);
   static void set_features_from(const char* march);
 
+  // Get information about cache line sizes.
+  // As of now and the foreseeable future, line size of all levels will be the same and 256.
+  static unsigned int Dcache_lineSize(unsigned int level = 0) { return _Dcache_lineSize; }
+  static unsigned int Icache_lineSize(unsigned int level = 0) { return _Icache_lineSize; }
+
+ public:
+
   // Get the CPU type from feature bit settings.
   static bool is_z900() { return has_long_displacement()      && !has_long_displacement_fast(); }
   static bool is_z990() { return has_long_displacement_fast() && !has_extended_immediate();  }
@@ -161,14 +179,9 @@ class VM_Version: public Abstract_VM_Version {
   static bool is_z10()  { return has_GnrlInstrExtensions()    && !has_DistinctOpnds(); }
   static bool is_z196() { return has_DistinctOpnds()          && !has_MiscInstrExt(); }
   static bool is_ec12() { return has_MiscInstrExt()           && !has_CryptoExt5(); }
-  static bool is_z13()  { return has_CryptoExt5();}
-
-  // Get information about cache line sizes.
-  // As of now and the foreseeable future, line size of all levels will be the same and 256.
-  static unsigned int Dcache_lineSize(unsigned int level = 0) { return _Dcache_lineSize; }
-  static unsigned int Icache_lineSize(unsigned int level = 0) { return _Icache_lineSize; }
-
- public:
+  static bool is_z13()  { return has_CryptoExt5()             && !has_MiscInstrExt2();}
+  static bool is_z14()  { return has_MiscInstrExt2()          && !has_MiscInstrExt3();}
+  static bool is_z15()  { return has_MiscInstrExt3();}
 
   // Need to use nested class with unscoped enum.
   // C++11 declaration "enum class Cipher { ... } is not supported.
@@ -265,32 +278,56 @@ class VM_Version: public Abstract_VM_Version {
   class MsgDigest {
     public:
       enum {
-        _Query            =   0,
-        _SHA1             =   1,
-        _SHA256           =   2,
-        _SHA512           =   3,
-        _GHASH            =  65,
-        _featureBits      = 128,
+        _Query                =   0,
+        _SHA1                 =   1,
+        _SHA256               =   2,
+        _SHA512               =   3,
+        _SHA3_224             =  32,
+        _SHA3_256             =  33,
+        _SHA3_384             =  34,
+        _SHA3_512             =  35,
+        _SHAKE_128            =  36,
+        _SHAKE_256            =  37,
+        _GHASH                =  65,
+        _featureBits          = 128,
 
         // Parameter block sizes (in bytes) for KIMD.
-        _Query_parmBlk_I  =  16,
-        _SHA1_parmBlk_I   =  20,
-        _SHA256_parmBlk_I =  32,
-        _SHA512_parmBlk_I =  64,
-        _GHASH_parmBlk_I  =  32,
+        _Query_parmBlk_I      =  16,
+        _SHA1_parmBlk_I       =  20,
+        _SHA256_parmBlk_I     =  32,
+        _SHA512_parmBlk_I     =  64,
+        _SHA3_224_parmBlk_I   = 200,
+        _SHA3_256_parmBlk_I   = 200,
+        _SHA3_384_parmBlk_I   = 200,
+        _SHA3_512_parmBlk_I   = 200,
+        _SHAKE_128_parmBlk_I  = 200,
+        _SHAKE_256_parmBlk_I  = 200,
+        _GHASH_parmBlk_I      =  32,
 
         // Parameter block sizes (in bytes) for KLMD.
-        _Query_parmBlk_L  =  16,
-        _SHA1_parmBlk_L   =  28,
-        _SHA256_parmBlk_L =  40,
-        _SHA512_parmBlk_L =  80,
+        _Query_parmBlk_L      =  16,
+        _SHA1_parmBlk_L       =  28,
+        _SHA256_parmBlk_L     =  40,
+        _SHA512_parmBlk_L     =  80,
+        _SHA3_224_parmBlk_L   = 200,
+        _SHA3_256_parmBlk_L   = 200,
+        _SHA3_384_parmBlk_L   = 200,
+        _SHA3_512_parmBlk_L   = 200,
+        _SHAKE_128_parmBlk_L  = 200,
+        _SHAKE_256_parmBlk_L  = 200,
 
         // Data block sizes (in bytes).
-        _Query_dataBlk    =   0,
-        _SHA1_dataBlk     =  64,
-        _SHA256_dataBlk   =  64,
-        _SHA512_dataBlk   = 128,
-        _GHASH_dataBlk    =  16
+        _Query_dataBlk        =   0,
+        _SHA1_dataBlk         =  64,
+        _SHA256_dataBlk       =  64,
+        _SHA512_dataBlk       = 128,
+        _SHA3_224_dataBlk     = 144,
+        _SHA3_256_dataBlk     = 136,
+        _SHA3_384_dataBlk     = 104,
+        _SHA3_512_dataBlk     =  72,
+        _SHAKE_128_dataBlk    = 168,
+        _SHAKE_256_dataBlk    = 136,
+        _GHASH_dataBlk        =  16
       };
   };
   class MsgAuthent {
@@ -394,11 +431,11 @@ class VM_Version: public Abstract_VM_Version {
   static bool has_HighWordInstr()             { return  (_features[0] & HighWordMask)                  == HighWordMask; }
   static bool has_FastSync()                  { return  (_features[0] & FastBCRSerializationMask)      == FastBCRSerializationMask; }
   static bool has_DistinctOpnds()             { return  (_features[0] & DistinctOpndsMask)             == DistinctOpndsMask; }
-  static bool has_CryptoExt3()                { return  (_features[1] & CryptoExtension3Mask)          == CryptoExtension3Mask; }
-  static bool has_CryptoExt4()                { return  (_features[1] & CryptoExtension4Mask)          == CryptoExtension4Mask; }
   static bool has_DFPZonedConversion()        { return  (_features[0] & DFPZonedConversionMask)        == DFPZonedConversionMask; }
   static bool has_DFPPackedConversion()       { return  (_features[1] & DFPPackedConversionMask)       == DFPPackedConversionMask; }
   static bool has_MiscInstrExt()              { return  (_features[0] & MiscInstrExtMask)              == MiscInstrExtMask; }
+  static bool has_MiscInstrExt2()             { return  (_features[0] & MiscInstrExt2Mask)             == MiscInstrExt2Mask; }
+  static bool has_MiscInstrExt3()             { return  (_features[0] & MiscInstrExt3Mask)             == MiscInstrExt3Mask; }
   static bool has_ExecutionHint()             { return  (_features[0] & ExecutionHintMask)             == ExecutionHintMask; }
   static bool has_LoadAndTrap()               { return  (_features[0] & LoadAndTrapMask)               == LoadAndTrapMask; }
   static bool has_ProcessorAssist()           { return  (_features[0] & ProcessorAssistMask)           == ProcessorAssistMask; }
@@ -406,21 +443,22 @@ class VM_Version: public Abstract_VM_Version {
   static bool has_LoadAndALUAtomicV2()        { return  (_features[0] & InterlockedAccess2Mask)        == InterlockedAccess2Mask; }
   static bool has_TxMem()                     { return ((_features[1] & TransactionalExecutionMask)    == TransactionalExecutionMask) &&
                                                        ((_features[0] & ConstrainedTxExecutionMask)    == ConstrainedTxExecutionMask); }
+  static bool has_CryptoExt3()                { return  (_features[1] & CryptoExtension3Mask)          == CryptoExtension3Mask; }
+  static bool has_CryptoExt4()                { return  (_features[1] & CryptoExtension4Mask)          == CryptoExtension4Mask; }
   static bool has_CryptoExt5()                { return  (_features[0] & CryptoExtension5Mask)          == CryptoExtension5Mask; }
+  static bool has_CryptoExt8()                { return  (_features[2] & CryptoExtension8Mask)          == CryptoExtension8Mask; }
+  static bool has_CryptoExt9()                { return  (_features[2] & CryptoExtension9Mask)          == CryptoExtension9Mask; }
   static bool has_LoadStoreConditional2()     { return  (_features[0] & LoadStoreConditional2Mask)     == LoadStoreConditional2Mask; }
   static bool has_VectorFacility()            { return  (_features[2] & VectorFacilityMask)            == VectorFacilityMask; }
-
-  static bool has_TestFeatureImpl()           { return  (_features[0] & TestFeature1ImplMask)          == TestFeature1ImplMask; }
-  static bool has_TestFeature1Impl()          { return  (_features[0] & TestFeature1ImplMask)          == TestFeature1ImplMask; }
-  static bool has_TestFeature2Impl()          { return  (_features[0] & TestFeature2ImplMask)          == TestFeature2ImplMask; }
-  static bool has_TestFeature4Impl()          { return  (_features[0] & TestFeature4ImplMask)          == TestFeature4ImplMask; }
-  static bool has_TestFeature8Impl()          { return  (_features[0] & TestFeature8ImplMask)          == TestFeature8ImplMask; }
-  static bool has_TestFeaturesImpl()          { return  has_TestFeature1Impl() || has_TestFeature2Impl() || has_TestFeature4Impl() || has_TestFeature8Impl(); }
+  static bool has_VectorEnhancements1()       { return  (_features[2] & VectorEnhancements1Mask)       == VectorEnhancements1Mask; }
+  static bool has_VectorEnhancements2()       { return  (_features[2] & VectorEnhancements2Mask)       == VectorEnhancements2Mask; }
+  static bool has_VectorPackedDecimal()       { return  (_features[2] & VectorPackedDecimalMask)       == VectorPackedDecimalMask; }
+  static bool has_VectorPackedDecimalEnh()    { return  (_features[2] & VectorPackedDecimalEnhMask)    == VectorPackedDecimalEnhMask; }
 
   // Crypto features query functions.
-  static bool has_Crypto_AES128()             { return has_Crypto() && test_feature_bit(&_cipher_features[0], Cipher::_AES128, Cipher::_featureBits); }
-  static bool has_Crypto_AES192()             { return has_Crypto() && test_feature_bit(&_cipher_features[0], Cipher::_AES192, Cipher::_featureBits); }
-  static bool has_Crypto_AES256()             { return has_Crypto() && test_feature_bit(&_cipher_features[0], Cipher::_AES256, Cipher::_featureBits); }
+  static bool has_Crypto_AES128()             { return has_Crypto() && test_feature_bit(&_cipher_features_KM[0], Cipher::_AES128, Cipher::_featureBits); }
+  static bool has_Crypto_AES192()             { return has_Crypto() && test_feature_bit(&_cipher_features_KM[0], Cipher::_AES192, Cipher::_featureBits); }
+  static bool has_Crypto_AES256()             { return has_Crypto() && test_feature_bit(&_cipher_features_KM[0], Cipher::_AES256, Cipher::_featureBits); }
   static bool has_Crypto_AES()                { return has_Crypto_AES128() || has_Crypto_AES192() || has_Crypto_AES256(); }
 
   static bool has_Crypto_SHA1()               { return has_Crypto() && test_feature_bit(&_msgdigest_features[0], MsgDigest::_SHA1,   MsgDigest::_featureBits); }
@@ -430,10 +468,6 @@ class VM_Version: public Abstract_VM_Version {
   static bool has_Crypto_SHA()                { return has_Crypto_SHA1() || has_Crypto_SHA256() || has_Crypto_SHA512() || has_Crypto_GHASH(); }
 
   // CPU feature setters (to force model-specific behaviour). Test/debugging only.
-  static void set_has_TestFeature1Impl()          { _features[0] |= TestFeature1ImplMask; }
-  static void set_has_TestFeature2Impl()          { _features[0] |= TestFeature2ImplMask; }
-  static void set_has_TestFeature4Impl()          { _features[0] |= TestFeature4ImplMask; }
-  static void set_has_TestFeature8Impl()          { _features[0] |= TestFeature8ImplMask; }
   static void set_has_DecimalFloatingPoint()      { _features[0] |= DecimalFloatingPointMask; }
   static void set_has_FPSupportEnhancements()     { _features[0] |= FPSupportEnhancementsMask; }
   static void set_has_ExecuteExtensions()         { _features[0] |= ExecuteExtensionsMask; }
@@ -468,15 +502,23 @@ class VM_Version: public Abstract_VM_Version {
   static void set_has_DistinctOpnds()             { _features[0] |= DistinctOpndsMask; }
   static void set_has_FPExtensions()              { _features[0] |= FPExtensionsMask; }
   static void set_has_MiscInstrExt()              { _features[0] |= MiscInstrExtMask; }
+  static void set_has_MiscInstrExt2()             { _features[0] |= MiscInstrExt2Mask; }
+  static void set_has_MiscInstrExt3()             { _features[0] |= MiscInstrExt3Mask; }
   static void set_has_ProcessorAssist()           { _features[0] |= ProcessorAssistMask; }
   static void set_has_InterlockedAccessV2()       { _features[0] |= InterlockedAccess2Mask; }
   static void set_has_LoadAndALUAtomicV2()        { _features[0] |= InterlockedAccess2Mask; }
   static void set_has_TxMem()                     { _features[0] |= ConstrainedTxExecutionMask; _features[1] |= TransactionalExecutionMask; }
+  static void set_has_LoadStoreConditional2()     { _features[0] |= LoadStoreConditional2Mask; }
   static void set_has_CryptoExt3()                { _features[1] |= CryptoExtension3Mask; }
   static void set_has_CryptoExt4()                { _features[1] |= CryptoExtension4Mask; }
-  static void set_has_LoadStoreConditional2()     { _features[0] |= LoadStoreConditional2Mask; }
   static void set_has_CryptoExt5()                { _features[0] |= CryptoExtension5Mask; }
+  static void set_has_CryptoExt8()                { _features[2] |= CryptoExtension8Mask; }
+  static void set_has_CryptoExt9()                { _features[2] |= CryptoExtension9Mask; }
   static void set_has_VectorFacility()            { _features[2] |= VectorFacilityMask; }
+  static void set_has_VectorEnhancements1()       { _features[2] |= VectorEnhancements1Mask; }
+  static void set_has_VectorEnhancements2()       { _features[2] |= VectorEnhancements2Mask; }
+  static void set_has_VectorPackedDecimal()       { _features[2] |= VectorPackedDecimalMask; }
+  static void set_has_VectorPackedDecimalEnh()    { _features[2] |= VectorPackedDecimalEnhMask; }
 
   static void reset_has_VectorFacility()          { _features[2] &= ~VectorFacilityMask; }
 
