@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -464,7 +464,7 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
                 Content inheritedTree = writer.getInheritedSummaryHeader(inheritedClass);
                 Content linksTree = writer.getInheritedSummaryLinksTree();
                 addSummaryFootNote(inheritedClass, inheritedMembers, linksTree, writer);
-                inheritedTree.addContent(linksTree);
+                inheritedTree.add(linksTree);
                 summaryTreeList.add(writer.getMemberTree(inheritedTree));
             }
         }
@@ -497,7 +497,7 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
             buildInheritedSummary(writer, kind, summaryTreeList);
         if (!summaryTreeList.isEmpty()) {
             Content memberTree = writer.getMemberSummaryHeader(typeElement, memberSummaryTree);
-            summaryTreeList.stream().forEach(memberTree::addContent);
+            summaryTreeList.stream().forEach(memberTree::add);
             writer.addMemberTree(memberSummaryTree, memberTree);
         }
     }
