@@ -31,13 +31,21 @@ package jdk.nashorn.api.tree;
  * <p><b>WARNING:</b> This interface and its sub-interfaces are
  * subject to change as the ECMAScript  programming language evolves.
  *
+ * @deprecated Nashorn JavaScript script engine and APIs, and the jjs tool
+ * are deprecated with the intent to remove them in a future release.
+ *
  * @since 9
  */
+@Deprecated(since="11", forRemoval=true)
 public interface Tree {
 
     /**
      * Enumerates all kinds of trees.
+     *
+     * @deprecated Nashorn JavaScript script engine and APIs, and the jjs tool
+     * are deprecated with the intent to remove them in a future release.
      */
+    @Deprecated(since="11", forRemoval=true)
     public enum Kind {
         /**
          * Used for instances of {@link ArrayAccessTree}.
