@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,20 +21,24 @@
  * questions.
  */
 
+package gc.g1;
+
 /**
  * @test TestShrinkAuxiliaryData15
  * @bug 8038423 8061715 8078405
+ * @key randomness
  * @summary Checks that decommitment occurs for JVM with different
  * G1ConcRSLogCacheSize and ObjectAlignmentInBytes options values
  * @requires vm.gc.G1
- * @requires vm.opt.AggressiveOpts=="false" | vm.opt.AggressiveOpts=="null"
+ * @requires vm.opt.AggressiveOpts != true
  * @library /test/lib
+ * @library /
  * @modules java.base/jdk.internal.misc
  *          java.management
  * @build sun.hotspot.WhiteBox
  * @run driver ClassFileInstaller sun.hotspot.WhiteBox
  *                              sun.hotspot.WhiteBox$WhiteBoxPermission
- * @run main/timeout=720 TestShrinkAuxiliaryData15
+ * @run main/timeout=720 gc.g1.TestShrinkAuxiliaryData15
  */
 public class TestShrinkAuxiliaryData15 {
 
