@@ -82,8 +82,9 @@ class EarlyReturnTarg {
     public static ClassLoader classLoaderValue;
     {
         try {
-            urls[0] = new URL("hi there");
-        } catch (java.net.MalformedURLException ee) {
+            urls[0] = new URL("file:/foo");
+        } catch (java.net.MalformedURLException ex) {
+            throw new AssertionError(ex);
         }
         classLoaderValue = new URLClassLoader(urls);
     }
@@ -116,8 +117,9 @@ class EarlyReturnTarg {
     public static ClassLoader eclassLoaderValue;
     {
         try {
-            urls[0] = new URL("been there, done that");
-        } catch (java.net.MalformedURLException ee) {
+            urls[0] = new URL("file:/bar");
+        } catch (java.net.MalformedURLException ex) {
+            throw new AssertionError(ex);
         }
         classLoaderValue = new URLClassLoader(urls);
     }
