@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,20 +26,15 @@
   @key headful
   @bug       6187066
   @summary   Tests the Window.autoRequestFocus property for the Window.setVisible() method.
-  @author    anton.tarasov: area=awt.focus
   @library    ../../regtesthelpers
   @build      Util
   @run       main AutoRequestFocusSetVisibleTest
 */
 
 import java.awt.*;
-import java.awt.event.*;
-import java.applet.Applet;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.lang.reflect.InvocationTargetException;
 import test.java.awt.regtesthelpers.Util;
 
-public class AutoRequestFocusSetVisibleTest extends Applet {
+public class AutoRequestFocusSetVisibleTest {
     static Frame focusedFrame;
     static Button focusOwner;
     static Frame frame;
@@ -65,10 +60,6 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
     }
 
     public void init() {
-        // Create instructions for the user here, as well as set up
-        // the environment -- set the layout manager, add buttons,
-        // etc.
-        this.setLayout (new BorderLayout ());
         toolkitClassName = Toolkit.getDefaultToolkit().getClass().getName();
     }
 
@@ -302,44 +293,40 @@ public class AutoRequestFocusSetVisibleTest extends Applet {
         // 6. Show unblocking modal Dialog.
         ///////////////////////////////////
 
-        if ("sun.awt.motif.MToolkit".equals(toolkitClassName)) {
-            System.out.println("Stage 6 - Skiping.");
-        } else {
-            System.out.println("Stage 6 in progress...");
+        System.out.println("Stage 6 in progress...");
 
-            // ---
-            // Testing the bug of activating invisible modal Dialog (awt_Window::SetAndActivateModalBlocker).
-            // Having some window not excluded from modality, so that it would be blocked.
-            Frame f = new Frame("Aux. Frame");
-            f.setSize(100, 100);
-            setVisible(f, true);
-            // ---
+        // ---
+        // Testing the bug of activating invisible modal Dialog (awt_Window::SetAndActivateModalBlocker).
+        // Having some window not excluded from modality, so that it would be blocked.
+        Frame f = new Frame("Aux. Frame");
+        f.setSize(100, 100);
+        setVisible(f, true);
+        // ---
 
-            setVisible(focusedFrame, true);
+        setVisible(focusedFrame, true);
+        if (!focusOwner.hasFocus()) {
+            Util.clickOnComp(focusOwner, robot);
+            Util.waitForIdle(robot);
             if (!focusOwner.hasFocus()) {
-                Util.clickOnComp(focusOwner, robot);
-                Util.waitForIdle(robot);
-                if (!focusOwner.hasFocus()) {
-                    throw new Error("Test error: the frame couldn't be focused.");
-                }
+                throw new Error("Test error: the frame couldn't be focused.");
             }
-
-            dialog.setModal(true);
-            dialog.setAutoRequestFocus(false);
-            focusedFrame.setModalExclusionType(Dialog.ModalExclusionType.APPLICATION_EXCLUDE);
-
-            TestHelper.invokeLaterAndWait(new Runnable() {
-                    public void run() {
-                        dialog.setVisible(true);
-                    }
-                }, robot);
-
-            if (dialog.isFocused()) {
-                throw new TestFailedException("the unblocking dialog shouldn't gain focus but it did!");
-            }
-            setVisible(dialog, false);
         }
 
+        dialog.setModal(true);
+        dialog.setAutoRequestFocus(false);
+        focusedFrame.setModalExclusionType(Dialog.ModalExclusionType.APPLICATION_EXCLUDE);
+
+        TestHelper.invokeLaterAndWait(new Runnable() {
+                public void run() {
+                    dialog.setVisible(true);
+                }
+            }, robot);
+
+        if (dialog.isFocused()) {
+            throw new TestFailedException("the unblocking dialog shouldn't gain focus but it did!");
+        }
+        setVisible(dialog, false);
+
         System.out.println("Test passed.");
     }
 
