@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -502,6 +502,8 @@ public abstract class CommandLineOptionTest {
             return "-minimal";
         } else if (Platform.isGraal()) {
             return "-graal";
+        } else if (Platform.isZero()) {
+            return "-zero";
         }
         throw new RuntimeException("Unknown VM mode.");
     }
