@@ -22,31 +22,39 @@
  */
 
 /**
- * Array extension check
+ * JDK-8035312_5 - pushes and pops for frozen array
  *
  * @test
  * @run
  */
 
-"use strict";
-var a = [1,2,3];
-Object.preventExtensions(a);
+var b = [1,2,3];
+Object.freeze(b);
+
 try {
-    a[4] = 4;
-    print(a);
+    b.push(4);
 } catch (e) {
-    if (!(e instanceof TypeError)) {
-	print("TypeError expected but got e");
+    print("length = " + b.length);
+    print("i caught an error"); 
+}
+print(b);
+print(b[3]);
+print("length = " + b.length);
+
+var c = [1,2,3];
+Object.freeze(c);
+
+for (var i = 0; i < 5; i++) {
+    try { 
+	c.pop();
+    } catch (e) { 
+	print("length = " + c.length);
+	print("I caught an error");
+	print(c);
     }
 }
 
-if (a[0] != 1) {
-    throw "element 0 is wrong";
-}
-if (a[1] != 2) {
-    throw "element 1 is wrong";
-}
-if (a[2] != 3) {
-    throw "element 2 is wrong";
-}
+print(c);
+print(c[3]);
+print("length = " + b.length);
 
